/* Driver program for date shifting */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

static void
usage(void)
{
    fprintf(stderr,
            "usage: dateshift [YYYY-MM-DD [HH:MM:SS]] [-c <command>]\n");
    fprintf(stderr, "Allows the modification of system time for command\n\n");
    fprintf(stderr, "Options:\n");
    fprintf(stderr, "\t-h\tThis help\n");
    fprintf(stderr, "\t-v\tVersion info\n");
    fprintf(stderr, "\t-c\t<command>\tCommand to run\n");
    fprintf(stderr, "\t-d\tDebug\n");
}

static void
show_version(void)
{
    fprintf(stderr, "%s version %s\n", PACKAGE, VERSION);
    fprintf(stderr, "Written by Alex Hornby <alex@hornby.org.uk>\n\n");
    fprintf(stderr, "This is free software; see the source for "
            "copying conditions.  There is NO\n"
            "warranty; not even for MERCHANTABILITY or FITNESS FOR A "
            "PARTICULAR PURPOSE\n");
}


static char *
find_intercept_lib (void)
{
    const char **dirname;
    char path[1024];
    char new_loc[2049];
    static const char *directories[] = {
        ".libs",
        ".",
        LIBDIR,
        NULL
    };
    char *lib_location;
        
    lib_location = NULL;
    for (dirname = directories; *dirname; ++dirname) {
        strcpy(path, *dirname);
        strcat(path, "/libdsintercept.so");
        if (!access (path, R_OK)) {
            lib_location = path;
            break;
        }
    }

    if (!lib_location) {
        fprintf(stderr, "Cannot find libdsintercept.so\n");
        exit(1);
    }

    /* Make lib_location absolute */
    if (lib_location[0] != '/') {
        getcwd(new_loc, 1024);
        strcat(new_loc, "/");
        strcat(new_loc, lib_location);
        lib_location = new_loc;
    }
    
    return strdup(lib_location);
}



int
start_command(int offset, const char* command)
{
    int status;

    if ( offset ) {
        char* envstr = malloc(2048);
        char *orig_ld_preload = getenv("LD_PRELOAD");
        char * lib_loc = find_intercept_lib();
        
        sprintf(envstr, "DS_OFFSET=%d", offset);
        putenv(envstr);

        envstr = malloc(2048);
        sprintf(envstr, "LD_PRELOAD=%s", lib_loc);
        if ( orig_ld_preload ) {
            strcat(envstr, ":");
            strcat(envstr, orig_ld_preload); 
        }
        putenv(envstr);
        free(lib_loc);
    }
    
    status = system(command);
    if ( status == -1 || status == 127) {
        fprintf(stderr, "error calling system\n");
        exit(1);
    }
    return status >> 8;
}

int
main(int argc, char** argv)
{
    int c = 0;
    char *command = 0;
    char *datetext = 0;
    struct tm origtime;
    struct tm newtime;
    time_t origtime_t;
    time_t newtime_t;
    char *dp = 0;
    int debug = 0;
    
    while(1) {
        c = getopt(argc, argv, "vdhc:");
        if ( c == -1 ) {
            break;
        }
        switch(c) {
        case 'h':
            usage();
            exit(0);
            break;
        case 'v':
            show_version();
            exit(0);
            break;
        case 'd':
            debug=1;
            break;
        case 'c':
            command = strdup(optarg);
            break;
        default:
            fprintf(stderr, "option error\n");
            usage();
            exit(1);
            break;
        }
    }

    if ( command == 0 ) {
        char *c = getenv("SHELL");
        if (c) {
            command = strdup(c);
        } else {
            command = strdup("/bin/sh");
        }
    }

    origtime_t = time(&origtime_t);
    origtime = *(localtime(&origtime_t));
    newtime = origtime;
    
    if (optind < argc)
    {
        int i;
        for(i = 0; optind < argc; ++i, ++optind) {
            if ( i == 0 ) {
                datetext = argv[optind];
                dp = strptime(datetext, "%Y-%m-%d", &newtime);
                if ( dp == NULL) {
                    fprintf(stderr, "Invalid date: %s\n", datetext);
                    exit(1);
                }
            } else {
                char *timetext = argv[optind];
                dp = strptime(timetext, "%H:%M:%S", &newtime);
                if ( dp == NULL ) {
                    fprintf(stderr, "Invalid date/time: %s\n", timetext);
                    exit(1);
                }
            }
        }
    } 
    newtime_t = mktime(&newtime);
    if ( debug ) {
        fprintf(stderr, "Time adjustment is %d seconds\n",
               (int) (newtime_t - origtime_t));
    }
    return start_command(newtime_t - origtime_t , command);
}
